# Scripts related to Documentation and Help.
# Author: K. Di Filippo

# Paths
PLUGIN_PATH= 'Virtualwind/'

# Global requires
require 'sketchup.rb'
Sketchup.load PLUGIN_PATH + 'vwGlobalConstants'

# Function to open index page for the VW help site.
def open_help_index

  # Get the VW_HOME path from the environment. (If VW_HOME doesn't 
  # return anything, make a best-guest attempt.)
  path = ENV['VW_HOME']
  if path == nil
    path = "c:\\Program Files\\Virtualwind\\Virtualwind Beta 2.0\\bin\\"
  end
  
  # Set the Bin and Documentation paths.
  bin_path = path
  profile_path = path + "..\\documentation\\"
  html_path = path + "..\\documentation\\UsersGuide\\"
  
  # Construct the command-line arguments.  (The file paths all have
  # to be in quotes, since it's possible there are spaces in the filenames.)
  command = "\"" + bin_path + "assistant.exe" + "\""
  profile_arguments = "-profile " + "\"" + profile_path + "VwHelp.adp" + "\""
  file_arguments = "-file " + "\"" + html_path + "Ch_Google_SketchUp_Tools.html" + "\""
  
  # Construct the command.
  commandLine = command + " " + profile_arguments + " " + file_arguments
  
  # Launch the help in its own Thread.  The separate Thread is
  # desirable since it allows the VW execution to be non-blocking.
  Thread.new do 
    
    # Change the working directory to the executable's path and execute.  
    # (This is encased in the equivalent of a try-catch block, to handle the 
    # case where the path is invalid.)
    begin
      system(commandLine)

    rescue SystemCallError
      UI::messagebox("Error: Could not find Virtualwind path (looked in " + path + ").  Please set the environment variable VW_HOME to the location where virtualwind.exe is located.")
    end
    
  end

end

# Function to open the "About..." box for VW
def open_about_vw
  about_message = "Virtualwind SketchUp Plugin\n\n"
  about_message += "Copyright Virtualwind Inc., 2007\n"
  about_message += 'www.virtualwind.com'
  UI.messagebox about_message
end

# Declare that the script file is loaded
file_loaded('documentationScripts.rb')
