# Plugin to launch Virtualwind from within SketchUp
# Author: K. Di Filippo

# Global requires
require 'sketchup.rb'

# Load dependencies
PLUGIN_PATH = 'Virtualwind/'
Sketchup.load PLUGIN_PATH + 'vwGlobalConstants'




# "Launch Observer" class.  This AppObserver class handles 
# Application-level events related to the Launching.  It should be 
# loaded by the VW Plugin Loader.  It works in conjuction with the
# Export Scripts and the VW Global Data Container.
class LaunchObserver < Sketchup::AppObserver
  
  
  # Called when the Model closes(?)  
  # (To be confirmed: SketchUp documentation is sketchy [no pun 
  # intended] on the details of this method.)
  def onClose()
  
     VW_GLOBAL_DATA_CONTAINER.lastExportedFile = nil
     
  end
  
  
  # Called when SketchUp creates a new, empty Model.
  def onNewModel(model)
  
     VW_GLOBAL_DATA_CONTAINER.lastExportedFile = nil
     
  end
  
  
  # Called when SketchUp opens a Model
  def onOpenModel(model)
  
     VW_GLOBAL_DATA_CONTAINER.lastExportedFile = nil
     
  end
  
  
  # Called when SketchUp quits.
  def onQuit()
  
     VW_GLOBAL_DATA_CONTAINER.lastExportedFile = nil
     
  end

  
end





# Launches an instance of Virtualwind in a separate Thread.
def launch_virtualwind

  # Check to see if the user has performed an Export, by looking up
  # the "Last Exported File" variable.  If not, display a warning
  # message and launch VW without any parameters.
  if VW_GLOBAL_DATA_CONTAINER.lastExportedFile == nil
     launchVWWithNoModel = UI.messagebox("No Export-to-VWS has been performed yet for this model.  Launch Virtualwind with no model?", MB_OKCANCEL, "Export-to-VWS not Performed")
     if(launchVWWithNoModel == 1)
     	commandLineArguments = ''
     else
        return
     end
  else
     commandLineArguments = "\"" + VW_GLOBAL_DATA_CONTAINER.lastExportedFile + "\""
  end


  # Launch Virtualwind in its own Thread.  The separate Thread is
  # desirable since it allows the VW execution to be non-blocking.
  Thread.new do 
    
    # Get the executable's path by looking up the VW_HOME 
    # environment variable.  If that doesn't return anything, make a
    # best-guess at what the executable's path might be.
    path = ENV['VW_HOME']
    if path == nil
      path = "c:\\Program Files\\Virtualwind\\Virtualwind 3.0\\bin"
    end
    
    # Change the working directory to the executable's path and execute.  
    # (This is encased in the equivalent of a try-catch block, to handle the 
    # case where the path is invalid.)
    begin
      Dir.chdir(path)
      commandLine = "Virtualwind " + commandLineArguments
      system(commandLine)
    rescue SystemCallError
      UI::messagebox("Error: Could not find Virtualwind path (looked in " + path + ").  Please set the environment variable VW_HOME to the location where virtualwind.exe is located.")
    end
    
  end
  
end

  
# Declare the file is loaded
file_loaded('virtualwindAppLauncher.rb')
