# Actual Loader Script for the VW Plug-in.
# Author: K. Di Filippo

# Path
PLUGIN_PATH= 'Virtualwind/'

# Global requires
require 'sketchup.rb'
Sketchup.load PLUGIN_PATH + 'vwGlobalConstants'
Sketchup.load PLUGIN_PATH + 'vwSketchupExtensions'
Sketchup.load PLUGIN_PATH + 'vwProgressBar'

# Load the individual plug-ins
Sketchup.load PLUGIN_PATH + 'geometryCorrectionScripts'
Sketchup.load PLUGIN_PATH + 'geometryValidationScripts'
Sketchup.load PLUGIN_PATH + 'virtualwindAppLauncher'
Sketchup.load PLUGIN_PATH + 'sewer'
Sketchup.load PLUGIN_PATH + 'exportScripts'
Sketchup.load PLUGIN_PATH + 'textureScripts'
Sketchup.load PLUGIN_PATH + 'utilityScripts'
Sketchup.load PLUGIN_PATH + 'documentationScripts'



# Update the menus
if( not file_loaded?('vwLoaderScript') )
  

  # Load VW Observers
  Sketchup.add_observer(LaunchObserver.new)


  # Create the commands
  
  # (Top-level commands)
  cmd_launch_vw = UI::Command.new('Launch Virtualwind') {launch_virtualwind}
  cmd_launch_vw.small_icon=PATH_ICONS + 'launchVW_small.png'
  cmd_launch_vw.large_icon=PATH_ICONS + 'launchVW_large.png'
  cmd_launch_vw.tooltip='Launch Virtualwind'
  cmd_launch_vw.status_bar_text='Launches Virtualwind application in a separate thread'


  # (Export commands)
  cmd_export_vws_all = UI::Command.new('Export to VWS format (all)') {export_vws_file EXPORT_MASK_ALL}
  cmd_export_vws_all.small_icon=PATH_ICONS + 'export_all_small.png'
  cmd_export_vws_all.large_icon=PATH_ICONS + 'export_all_large.png'
  cmd_export_vws_all.tooltip='Export to VWS format (all)'
  cmd_export_vws_all.status_bar_text='Exports all Solid and Canopy geometry to a VWS-format file.'

  cmd_export_vws_model_as_sheets = UI::Command.new('Export to VWS format (all as Canopies)') {export_vws_file EXPORT_MASK_FORCE_SHEETS + EXPORT_MASK_ALL}
  cmd_export_vws_model_as_sheets.small_icon=PATH_ICONS + 'export_all_as_canopies_small.png'
  cmd_export_vws_model_as_sheets.large_icon=PATH_ICONS + 'export_all_as_canopies_large.png'
  cmd_export_vws_model_as_sheets.tooltip='Export to VWS format (all as Canopies)'
  cmd_export_vws_model_as_sheets.status_bar_text='Exports all geometry to a VWS-format file as Canopies geometry.'

  cmd_export_vws_solids = UI::Command.new('Export to VWS format (Solids)') {export_vws_file EXPORT_MASK_SOLIDS}
  cmd_export_vws_solids.small_icon=PATH_ICONS + 'export_buildings_small.png'
  cmd_export_vws_solids.large_icon=PATH_ICONS + 'export_buildings_large.png'
  cmd_export_vws_solids.tooltip='Export to VWS format (Solids)'
  cmd_export_vws_solids.status_bar_text='Exports only solid geometry to a VWS-format file (Canopy geometry is ignored).'

  cmd_export_vws_sheets = UI::Command.new('Export to VWS format (Canopies)') {export_vws_file EXPORT_MASK_SHEETS}
  cmd_export_vws_sheets.small_icon=PATH_ICONS + 'export_canopies_small.png'
  cmd_export_vws_sheets.large_icon=PATH_ICONS + 'export_canopies_large.png'
  cmd_export_vws_sheets.tooltip='Export to VWS format (Canopies)'
  cmd_export_vws_sheets.status_bar_text='Exports only Canopy geometry to a VWS-format file (solid geometry is ignored).'

  cmd_export_without_textures = UI::Command.new('Export With Textures') {toggleExportWithTextures}
  cmd_export_without_textures.small_icon=PATH_ICONS + 'export_with_textures_small.png'
  cmd_export_without_textures.large_icon=PATH_ICONS + 'export_with_textures_large.png'
  cmd_export_without_textures.tooltip='Export With Textures'
  cmd_export_without_textures.status_bar_text='Export With Textures'
  cmd_export_without_textures.set_validation_proc{
  	if VW_GLOBAL_DATA_CONTAINER.exportWithTextures?
  		MF_CHECKED
  	else
  		MF_UNCHECKED
  	end
  }

  
  # (Utility commands)
  cmd_utility_unhide_all = UI::Command.new('Display all') {unhide_all_entities}
  cmd_utility_unhide_all.small_icon=PATH_ICONS + 'display_all_small.png'
  cmd_utility_unhide_all.large_icon=PATH_ICONS + 'display_all_large.png'
  cmd_utility_unhide_all.tooltip='Display all'
  cmd_utility_unhide_all.status_bar_text="Displays all Entities in a model (subject to settings in the \"View\" menu)."

  cmd_utility_separate_solids_and_sheets = UI::Command.new('Separate Solids and Canopies') {separate_solids_and_sheets Sketchup.active_model.entities}
  cmd_utility_separate_solids_and_sheets.small_icon=PATH_ICONS + 'separate_buildings_and_canopies_small.png'
  cmd_utility_separate_solids_and_sheets.large_icon=PATH_ICONS + 'separate_buildings_and_canopies_large.png'
  cmd_utility_separate_solids_and_sheets.tooltip='Separate Solids and Canopies'
  cmd_utility_separate_solids_and_sheets.status_bar_text='Separates Solid objects from Canopies.'

  cmd_utility_make_terrain = UI::Command.new('Make into Terrain') {make_selection_into_terrain}
  cmd_utility_make_terrain.small_icon=PATH_ICONS + 'designate_selection_as_terrain_small.png'
  cmd_utility_make_terrain.large_icon=PATH_ICONS + 'designate_selection_as_terrain_large.png'
  cmd_utility_make_terrain.tooltip='Make into Terrain'
  cmd_utility_make_terrain.status_bar_text="Designates the Selection as Terrain."


  # (Watertightness commands)
  cmd_watertight_show_nonwatertight = UI::Command.new('Show non-Watertight geometry') {show_invalid_entities Sketchup.active_model.entities}
  cmd_watertight_show_nonwatertight.small_icon=PATH_ICONS + 'hide_valid_geometry_small.png'
  cmd_watertight_show_nonwatertight.large_icon=PATH_ICONS + 'hide_valid_geometry_large.png'
  cmd_watertight_show_nonwatertight.tooltip='Show non-Watertight geometry'
  cmd_watertight_show_nonwatertight.status_bar_text='Hides all geometry in the model that is Watertight.'
  
  cmd_watertight_check_all = UI::Command.new('Check for Watertightness (All)') {validate_entities Sketchup.active_model.entities}
  cmd_watertight_check_all.small_icon=PATH_ICONS + 'validate_all_geometry_small.png'
  cmd_watertight_check_all.large_icon=PATH_ICONS + 'validate_all_geometry_large.png'
  cmd_watertight_check_all.tooltip='Check for Watertightness (All)'
  cmd_watertight_check_all.status_bar_text='Checks all geometry for Watertightness, and highlights the first violation encountered.'

  cmd_watertight_check_active = UI::Command.new('Check for Watertightness (Active)') {validate_entities Sketchup.active_model.active_entities}
  cmd_watertight_check_active.small_icon=PATH_ICONS + 'validate_active_geometry_small.png'
  cmd_watertight_check_active.large_icon=PATH_ICONS + 'validate_active_geometry_large.png'
  cmd_watertight_check_active.tooltip='Check for Watertightness (Active)'
  cmd_watertight_check_active.status_bar_text='Checks the active geometry for Watertightness, and highlights the first violation encountered.'
  
  cmd_watertight_check_selected = UI::Command.new('Check for Watertightness (Selected)') {validate_entities Sketchup.active_model.selection}
  cmd_watertight_check_selected.small_icon=PATH_ICONS + 'validate_selected_geometry_small.png'
  cmd_watertight_check_selected.large_icon=PATH_ICONS + 'validate_selected_geometry_large.png'
  cmd_watertight_check_selected.tooltip='Check for Watertightness (Selected)'
  cmd_watertight_check_selected.status_bar_text='Checks selected geometry for Watertightness, and highlights the first violation encountered.'
  
  cmd_watertight_compute_stats = UI::Command.new('Compute Watertightness statistics') {show_validation_statistics Sketchup.active_model.active_entities}
  cmd_watertight_compute_stats.small_icon=PATH_ICONS + 'compute_validation_statistics_small.png'
  cmd_watertight_compute_stats.large_icon=PATH_ICONS + 'compute_validation_statistics_large.png'
  cmd_watertight_compute_stats.tooltip='Compute Watertightness statistics'
  cmd_watertight_compute_stats.status_bar_text='Computes the Watertightness statistics of the active geometry.'


  # (Texture commands)
  cmd_validate_textures_no_problem = UI::Command.new('Detect valid textures') {detect_valid_textures}
  cmd_validate_textures_no_problem.small_icon=PATH_ICONS + 'detect_valid_textures_small.png'
  cmd_validate_textures_no_problem.large_icon=PATH_ICONS + 'detect_valid_textures_large.png'
  cmd_validate_textures_no_problem.tooltip='Detect valid textures'
  cmd_validate_textures_no_problem.status_bar_text='Detects valid textures within the entire model.'

  cmd_validate_textures_any_problem = UI::Command.new('Detect problematic textures (any)') {detect_problematic_textures TEXTURE_PROBLEM_TYPE_ANY_PROBLEM}
  cmd_validate_textures_any_problem.small_icon=PATH_ICONS + 'detect_invalid_textures_small.png'
  cmd_validate_textures_any_problem.large_icon=PATH_ICONS + 'detect_invalid_textures_large.png'
  cmd_validate_textures_any_problem.tooltip='Detect problematic textures (any)'
  cmd_validate_textures_any_problem.status_bar_text='Detects problematic textures within the entire model (any problem).'

  cmd_validate_textures_front_back_different = UI::Command.new('Detect problematic textures (back texture different from front)') {detect_problematic_textures TEXTURE_PROBLEM_TYPE_TWO_DIFFERENT_TEXTURES}
  cmd_validate_textures_front_back_different.small_icon=PATH_ICONS + 'detect_invalid_back_different_than_front_small.png'
  cmd_validate_textures_front_back_different.large_icon=PATH_ICONS + 'detect_invalid_back_different_than_front_large.png'
  cmd_validate_textures_front_back_different.tooltip='Detect problematic textures (back texture different from front)'
  cmd_validate_textures_front_back_different.status_bar_text='Detects problematic textures within the entire model (back texture different from front).'

  cmd_validate_textures_back_without_front = UI::Command.new('Detect problematic textures (back without front)') {detect_problematic_textures TEXTURE_PROBLEM_TYPE_BACK_TEXTURE_WITHOUT_FRONT_TEXTURE}
  cmd_validate_textures_back_without_front.small_icon=PATH_ICONS + 'detect_invalid_back_without_front_small.png'
  cmd_validate_textures_back_without_front.large_icon=PATH_ICONS + 'detect_invalid_back_without_front_large.png'
  cmd_validate_textures_back_without_front.tooltip='Detect problematic textures (back without front)'
  cmd_validate_textures_back_without_front.status_bar_text='Detects problematic textures within the entire model (back without front).'

  
  # (Correction commands)
  cmd_correction_remove_zero_faced_edges = UI::Command.new('Remove zero-faced Edges') {remove_zero_faced_edges}
  cmd_correction_remove_zero_faced_edges.small_icon=PATH_ICONS + 'remove_edges_with_no_faces_small.png'
  cmd_correction_remove_zero_faced_edges.large_icon=PATH_ICONS + 'remove_edges_with_no_faces_large.png'
  cmd_correction_remove_zero_faced_edges.tooltip='Remove zero-faced Edges'
  cmd_correction_remove_zero_faced_edges.status_bar_text='Removes all Edges in model with no attached Faces.'

  cmd_correction_flip_faces_textured_back_without_front = UI::Command.new('Flip back-only Textured Faces') {reverse_orientation_and_texture_for_backtextured_faces}
  cmd_correction_flip_faces_textured_back_without_front.small_icon=PATH_ICONS + 'fix_back_without_front_small.png'
  cmd_correction_flip_faces_textured_back_without_front.large_icon=PATH_ICONS + 'fix_back_without_front_large.png'
  cmd_correction_flip_faces_textured_back_without_front.tooltip='Flip back-only Textured Faces'
  cmd_correction_flip_faces_textured_back_without_front.status_bar_text='Flips the orientation and texture-face of all Faces in the model that have a back Texture without a front Texture.'

  cmd_correction_sew_edges = UI::Command.new('Sew Edges') {Sketchup.active_model.select_tool EdgeSewer.new}
  cmd_correction_sew_edges.small_icon=PATH_ICONS + 'sew_two_edges_small.png'
  cmd_correction_sew_edges.large_icon=PATH_ICONS + 'sew_two_edges_large.png'
  cmd_correction_sew_edges.tooltip='Sew Edges'
  cmd_correction_sew_edges.status_bar_text='Sews two Edges together by adding a Face-triangulation between them.'

  # (Help commands)
  cmd_help_home = UI::Command.new('Virtualwind Plugin Help') {open_help_index}
  cmd_help_home.status_bar_text='Opens Help site for the Virtualwind Plugin.'

  cmd_help_about_vw = UI::Command.new('About Virtualwind Plugin....') {open_about_vw}
  cmd_help_about_vw.status_bar_text="Opens 'About' information for the Virtualwind Plugin."


  # "Create" the Plugins menu
  plugins_menu = UI.menu('Plugins')
  
  # Create and populate the Virtualwind submenu
  vw_menu = plugins_menu.add_submenu('Virtualwind')
  vw_menu.add_item cmd_launch_vw
  
  # Create and populate the Export sub-submenu
  export_submenu = vw_menu.add_submenu('Export tools')
  export_submenu.add_item cmd_export_vws_all
  export_submenu.add_item cmd_export_vws_model_as_sheets
  export_submenu.add_item cmd_export_vws_solids
  export_submenu.add_item cmd_export_vws_sheets
  export_submenu.add_item cmd_export_without_textures

  # Create and populate the Utility sub-submenu
  utility_submenu = vw_menu.add_submenu('Utility tools')
  utility_submenu.add_item cmd_utility_unhide_all
  utility_submenu.add_item cmd_utility_separate_solids_and_sheets
  utility_submenu.add_item cmd_utility_make_terrain

  # Create and populate the Watertightness sub-submenu
  watertightness_submenu = vw_menu.add_submenu('Watertightness tools')
  watertightness_submenu.add_item cmd_watertight_show_nonwatertight
  watertightness_submenu.add_item cmd_watertight_check_all
  watertightness_submenu.add_item cmd_watertight_check_active
  watertightness_submenu.add_item cmd_watertight_check_selected
  watertightness_submenu.add_item cmd_watertight_compute_stats
  
  # Create and populate the Textures sub-submenu
  textures_submenu = vw_menu.add_submenu('Texture tools')
  textures_submenu.add_item cmd_validate_textures_no_problem
  textures_submenu.add_item cmd_validate_textures_any_problem
  textures_submenu.add_item cmd_validate_textures_front_back_different
  textures_submenu.add_item cmd_validate_textures_back_without_front
  
  # Create and populate the Correction sub-submenu
  correction_submenu = vw_menu.add_submenu('Correction tools')
  correction_submenu.add_item cmd_correction_remove_zero_faced_edges
  correction_submenu.add_item cmd_correction_flip_faces_textured_back_without_front
  correction_submenu.add_item cmd_correction_sew_edges
  
  # Get the Help menu and populate VW-related Help Topics
  help_menu = UI.menu('Help')
  vw_help_menu = help_menu.add_submenu('Virtualwind Plugin')
  vw_help_menu.add_item cmd_help_home
  vw_help_menu.add_item cmd_help_about_vw
  
  
  # Create and populate the Toolbar
  vw_toolbar = UI::Toolbar.new 'Virtualwind Toolbar'

  vw_toolbar.add_item cmd_launch_vw
  vw_toolbar.add_separator

  vw_toolbar.add_item cmd_export_vws_all
  vw_toolbar.add_item cmd_export_vws_model_as_sheets
  vw_toolbar.add_item cmd_export_vws_solids
  vw_toolbar.add_item cmd_export_vws_sheets
  vw_toolbar.add_item cmd_export_without_textures
  
  vw_toolbar.add_separator
  vw_toolbar.add_item cmd_utility_unhide_all
  vw_toolbar.add_item cmd_utility_separate_solids_and_sheets
  vw_toolbar.add_item cmd_utility_make_terrain

  vw_toolbar.add_separator
  vw_toolbar.add_item cmd_watertight_show_nonwatertight
  vw_toolbar.add_item cmd_watertight_check_all
  vw_toolbar.add_item cmd_watertight_check_active
  vw_toolbar.add_item cmd_watertight_check_selected
  vw_toolbar.add_item cmd_watertight_compute_stats

  vw_toolbar.add_separator
  vw_toolbar.add_item cmd_validate_textures_no_problem
  vw_toolbar.add_item cmd_validate_textures_any_problem
  vw_toolbar.add_item cmd_validate_textures_front_back_different
  vw_toolbar.add_item cmd_validate_textures_back_without_front
  
  vw_toolbar.add_separator
  vw_toolbar.add_item cmd_correction_remove_zero_faced_edges
  vw_toolbar.add_item cmd_correction_flip_faces_textured_back_without_front
  vw_toolbar.add_item cmd_correction_sew_edges


  # Tell the toolbar to show itself.
  vw_toolbar.show
  
  
end

# Declare the file is loaded
file_loaded('vwLoaderScript')
